<?PHP @template::ran_directly(); /* required for all templates */ 

/**
 * 3rd Party Module Code Hooks
 *
 * @package    3rd_Party_Module_Code_Hooks
 * @author     Andy Rockwell <support@solidphp.com>
 */

/**
* Runs just before a new license instance is consumed by the SPBAS method.
* This hook will interrupt runtime if you return anything other than false.
* So for example, if you returned 'Error: This happened before a new instance was consumed.'
* then the end user would be presented with that error message.
* 
* @param array $module
* @param integer $license_id
* @param string $license_key
* @param array $license
* @param array $method_attributes
* @param array $method_attributes_override
* @param array $instances
* @return boolean/string
*/
function Example_license_validation_before_new_instance_added($module, $license_id, $license_key, $license, $method_attributes, $method_attributes_override, $instances)
	{
	return false;
	}

/**
* Runs just before a license key is created.
* 
* @param array $module
* @param array $license_array
* @return boolean/string
*/
function Example_at_license_key_creation($module, $license_array)
	{
	return false;
	}

/**
* Runs just before a user is deleted
* 
* @param array $module
* @param integer $user_id
* @return boolean 
*/
function Example_admin_before_user_delete($module, $user_id)
	{
	return true;
	}

/**
* Runs just before a user is deleted from the helpdesk
* 
* @param array $module
* @param integer $user_id
* @return boolean 
*/
function Example_helpdesk_before_user_delete($module, $user_id)
	{
	return true;
	}

/**
* Runs just after a package has been added
* 
* @param array $module
* @param array $flagged_array
* @param array $license
* @param array $license_array
* @return boolean 
*/
function Example_at_phpaudit_license_invalid($module, $flagged_array, $license, $license_array)
	{
	return true;
	}

/**
* Runs just after a package has been added
* 
* @param array $module
* @return boolean 
*/
function Example_at_api_request($module)
	{
	return true;
	}

/**
* Runs just after a package has been added
* 
* @param array $module
* @param integer $package_id
* @return boolean 
*/
function Example_after_package_creation($module, $package_id)
	{
	return true;
	}

/**
* Runs when a new user is registered from a support ticket import via cron
* 
* @param array $module
* @param integer $license_id
* @param string $license_key
* @param integer $licensing_method_id
* @param integer $customer_id
* @param integer $package_id
* @param integer $product_id
* @param integer $sku_id
* @param string $status
* @param mixed $expires
* @param mixed $local_expires
* @param mixed $activated
* @param integer $created
* @return boolean 
*/
function Example_after_license_key_creation($module, $license_id, $license_key, $licensing_method_id, $customer_id, $package_id, $product_id, $sku_id, $status, $expires, $local_expires, $activated, $created)
	{
	return true;
	}

/**
* Runs when a new user is registered from a support ticket import via cron
* 
* @param array $module Module specific data.
* @param string $default_item_name
* @param array $all_available_data
* @return string 
*/
function Example_before_payment_item_name_sent_to_processor($module, $default_item_name, $all_available_data)
	{
	return $default_item_name;
	}

/**
* Runs when a new user is registered from a support ticket import via cron
* 
* @param array $module Module specific data.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @return array 
*/
function Example_helpdesk_create_new_ticket_new_user_registered($module, $user_id, $first_name, $last_name, $email, $username, $password)
	{
	return array('display' => '');
	}

/**
* Runs when a new user is registered from a support ticket import via cron
* 
* @param array $module Module specific data.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @return array 
*/
function Example_helpdesk_import_new_user_registered($module, $user_id, $first_name, $last_name, $email, $username, $password)
	{
	return array('display' => '');
	}

/**
* Runs when an order is processed to completion from the order system.
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $order_id
* @return array $third_party available to order/compiler/order_complete.php & order/templates/default/order_complete.php
*/
function Example_order_form_order_complete($module, $buffer, $order_id)
	{
	// don't run the order complete modules twice
	if (third_party::isset_log($module['id'], 'order_id', $order_id)) { return false; }

	// set the order ID to ensure the modules aren't run twice
	third_party::log($module['id'], 'order_id', $order_id);

	return array('display' => '');
	}

/**
* Runs when a new user is registered from the customer area (?task=register).
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @param integer $customer_id Only populated when Admin -> Settings -> Storefront & Products, Customer Areas -> Advanced -> Create Custoemr Account w/ Registration == Yes
* @return array $third_party available to customers/compiler/register.php & customers/templates/default/register.php
*/
function Example_customer_area_new_user_registered($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $password, $customer_id=0)
	{
	return array('display' => '');
	}

/**
* Runs when a new user is registered from the order system.
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param integer $customer_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @return array $third_party available to order/compiler/checkout.php & order/templates/default/checkout.php
*/
function Example_order_form_new_user_registered($module, $buffer, $user_id, $customer_id, $first_name, $last_name, $email, $username, $password)
	{
	return array('display' => '');
	}

/**
* Runs when a new user is registered from the admin area.
* 
* @param array $module Module specific data.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @return boolean
*/
function Example_admin_new_user_registered($module, $user_id, $first_name, $last_name, $email, $username, $password)
	{
	return true;
	}

/**
* Runs when user information is updated at checkout in the order system (?task=checkout).
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param integer $customer_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return array $third_party available to order/compiler/checkout.php & order/templates/default/checkout.php
*/
function Example_order_form_update_user($module, $buffer, $user_id, $customer_id, $first_name, $last_name, $email, $username, $password, $hashed_password)
	{
	return array('display' => '');
	}

/**
* Runs when user information is updated by the logged in customer (?task=my_account)
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param integer $customer_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return array $third_party available to customers/compiler/my_account.php & customers/templates/default/my_account.php
*/
function Example_customer_area_update_user($module, $buffer, $user_id, $customer_id, $first_name, $last_name, $email, $username, $status, $password, $hashed_password)
	{
	return array('display' => '');
	}

/**
* Runs when the admin updates user information from the admin area.
* 
* @param array $module Module specific data.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return boolean
*/
function Example_admin_update_user($module, $user_id, $first_name, $last_name, $email, $username, $status, $password, $hashed_password)
	{
	return true;
	}

/**
* Runs when the user logs in at checkout in the order system (?task=checkout)
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @return array $third_party available to order/compiler/checkout.php & order/templates/default/checkout.php
*/
function Example_order_form_user_login($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $status, $password)
	{
	return array('display' => '');
	}

/**
* Runs when the user logs in at checkout in the order system (?task=checkout)
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @return array $third_party available to customers/compiler/login.php & customers/templates/default/login.php
*/
function Example_customer_area_user_login($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $status, $password)
	{
	return array('display' => '');
	}

/**
* Runs when the user logs in at checkout in the order system (?task=checkout)
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @return array $third_party available to order/compiler/checkout.php & order/templates/default/checkout.php
*/
function Example_order_form_resend_password($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $status, $password)
	{
	return array('display' => '');
	}

/**
* Runs when the user logs in at checkout in the order system (?task=checkout)
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return array $third_party available to customers/compiler/login.php & customers/templates/default/login.php
*/
function Example_customer_area_resend_password($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $status, $password, $hashed_password)
	{
	return array('display' => '');
	}

/**
* Runs when the user changes there password from the customer area (?task=my_account&tab=change_password).
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $status enabled or disabled
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return array $third_party available to customers/compiler/my_account.php & customers/templates/default/my_account.php
*/
function Example_customer_area_change_password($module, $buffer, $user_id, $first_name, $last_name, $email, $username, $status, $password, $hashed_password)
	{
	return array('display' => '');
	}

/**
* Runs when the user updates their information from the customer area (?task=my_account&tab=my_info).
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param integer $customer_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @param string $hashed_password md5('plain text password')
* @return array $third_party available to customers/compiler/my_account.php & customers/templates/default/my_account.php
*/
function Example_customer_area_update_my_info($module, $buffer, $user_id, $customer_id, $first_name, $last_name, $email, $username, $password, $hashed_password)
	{
	return array('display' => '');
	}

/**
* Runs when the user changes there password from the customer area (?task=my_account&tab=change_password).
* 
* @param array $module Module specific data.
* @param array $buffer All data available.
* @param integer $user_id
* @param integer $customer_id
* @param string $first_name
* @param string $last_name
* @param string $email
* @param string $username
* @param string $password
* @return array $third_party available to customers/compiler/my_account.php & customers/templates/default/my_account.php
*/
function Example_customer_area_add_contact($module, $buffer, $user_id, $customer_id, $first_name, $last_name, $email, $username, $password)
	{
	return array('display' => '');
	}
?>