<?PHP

/**
 * SPBAS General Use Functions
 *
 * @package    General_Use_Functions
 * @author     Andy Rockwell <support@solidphp.com>
 */

/**
* Generate a random hex color
* 
* @return string
*/
function detect_browser_language()
	{
	foreach (explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']) as $lang) 
		{
		$pattern='/^(?P<primarytag>[a-zA-Z]{2,8})(?:-(?P<subtag>[a-zA-Z]{2,8}))?(?:(?:;q=)(?P<quantifier>\d\.\d))?$/';
		$splits=array();

		if (preg_match($pattern, $lang, $splits)) 
			{
			return strtolower($splits[0]);
			}
		}

	return 'en-us';
	}

/**
* Display the long date format in the top navigation bar of admin
* 
* @return string
*/
function top_nav_date()
	{
	$saved_format=config('date_format_long');
	// $date_format_long=($saved_format)?$saved_format:'l F jS, Y';
	$date_format_long=($saved_format&&!is_array($saved_format))?$saved_format:'l F jS, Y';

	return date($date_format_long);
	}

/**
* Force the use of SSL 
* 
* @param string $redirect_url
* @return mixed
*/
function force_ssl($redirect_url)
	{
	if (!x('task')&&x('mod')&&x('mod')=='cron_jobs') { return false; }
	if (config('force_ssl')=='Yes'&&!isset($_SERVER['HTTPS'])) 
		{
		$redirect_url=sts(strtolower($redirect_url));
		list($http, $redirect_url)=explode('://', $redirect_url);
		header("Location: https://{$redirect_url}/");
		exit;
		}

	return true;
	}

/**
* Generate a random hex color
* 
* @return string
*/
function get_random_color()
	{
	for ($i = 0; $i<6; $i++)
		{
		$c.= dechex(rand(0,15));
		}

	return $c;
	} 

/**
* Determine if we are using windows
* 
* @param string $path
* @return boolean
*/
function is_windows($path='')
	{
	if (@strpos($path, ':\\')!==false) { return true; }
	if (substr(php_uname(), 0, 7)=='Windows') { return true; }
	return false; 
	}

/**
* Post XML using cURL
* 
* @param integer $url
* @param integer $xml
* @return string
*/
function post_xml($url, $xml) 
	{
	$ch=curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: text/xml"));
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
	$result=curl_exec($ch);
	curl_close($ch);

	return $result;
	}

/**
* Is the media upload path set yet?
*
* @param string $media_upload_path
* @return boolean
*/
function isset_media_upload_path($media_upload_path=false)
	{
	$media_upload_path=($media_upload_path)?$media_upload_path:config('media_upload_path');
	if (!$media_upload_path||is_array($media_upload_path)) { return false; }

	clearstatcache();
	if (!@is_dir($media_upload_path)) { return false; }
	if (!@is_writable($media_upload_path)) { return false; }

	return true;
	}


/**
* Button template
*
* @param string $link_out
* @param string $text
* @param string $icon
* @param string $js
* @return string
*/
function bt($link_out, $text, $icon='add', $js='')
	{
	return "<a class='button'{$js} href='{$link_out}'>"
			."{$text}"
			."<img src='view/images/button_right_{$icon}.gif' /></a>";
	}

/**
* Set the default tab on page refresh
*
* @param string $default_tab
* @return string
*/
function set_default_tab($default_tab)
	{
	if (x('show_tab')) { $default_tab=x('show_tab'); }

	return "<script> toggle_tabs('{$default_tab}', '".x('mod')."', '".x('task')."'); </script>";
	}

/**
* Parse API home URL
*
* @return string
*/
function api_home()
	{
	$api_home=config('api_home');
	$api_home=explode('/', $api_home);
	array_pop($api_home);
	return implode('/', $api_home).'/api';
	}

/**
* Get $base + or - 1
*
* @param string $table
* @param string $column
* @param string $direction
* @param integer $base
* @return integer|boolean integer auto-inc ID on success; boolean false on failure
*/
function flip_page($table, $column, $direction, $base)
	{
	$asc_desc=($direction=='>')?'asc':'desc';

	$rs=mysql_fetch_array(mysql_query("select `{$column}` from `{$table}` where `{$column}`{$direction}{$base} order by `{$column}` {$asc_desc} limit 0, 1"));

	return ($rs['id'])?$rs['id']:false;
	}

/**
* Figure out the time that has passed from timestamp A to B
*
* @param integer $seconds Time in seconds
* @return string|boolean string on success; boolean false on failure
*/
function ago($seconds)
	{
	// seconds
	if ($seconds<60&&$seconds>0)
		{
		return $seconds.' '.w('seconds_ago', 0);
		}

	// minutes
	elseif ($seconds<3600&&$seconds>60)
		{
		$ago=round($seconds/60);
		return "{$ago} ".(($ago==1)?w('minute', 0):w('minutes', 0)).' '.w('ago', 0);
		}

	// hours
	elseif ($seconds<86400&&$seconds>3600)
		{
		$ago=round($seconds/3600);
		return "{$ago} ".(($ago==1)?w('hour', 0):w('hours', 0)).' '.w('ago', 0);
		}

	// days
	elseif ($seconds<2678400&&$seconds>86400)
		{
		$ago=$seconds/86400;

		if ($ago<=2) 
			{ 
			return w('Yesterday', 0); 
			}

		return round($ago).' '.w('days_ago', 0);
		}

	return false;
	}

/**
* Make file names a-z A-Z and 0-9 only, with the exception of _
*
* @param string $file_name
* @return string
*/
function clean_filename($file_name)
	{
	$file_name=str_replace(" ", '_', $file_name);
	$file_name=str_replace("..", '_', $file_name);
	return ereg_replace("[^A-Za-z0-9\-\_\.]", '', $file_name);
	}

/**
* Convert a Comma Separated Value list into an array
* 
* @param string $path
* @return array
*/
function parse_sql($path)
	{
	$buffer='';
	$results=array();
	$source=file($path);
	foreach ($source as $sql)
		{
		$sql=trim($sql);
		if (substr($sql, 0, 1)==" ") { continue; }
		if (substr($sql, 0, 2)=="--") { continue; }
		if (!$sql) { continue; }

		if (substr($sql, -1)==";")
			{
			$buffer.=$sql;
			$buffer=substr($buffer, 0, strlen($buffer)-1);
			# $buffer=str_replace("`", "", $buffer);

			$results[]=$buffer;
			$buffer='';
			}
		else
			{
			$buffer.=$sql;
			}
		}

	return $results;
	}

/**
* Convert a Comma Separated Value list into an array
* 
* @param string $csv_string
* @return array
*/
function csv2array($csv_string)
	{
	return explode(',', $csv_string);
	}

/**
* CSS automation, make a row an alt row or not.
* 
* @param string $var
* @param boolean $which
* @return string
*/
function alt($var, $which=false)
	{
	if ($which)
		{
		return (!ooe($var))?"class='alt'":'';
		}

	return (!ooe($var))?'_alt':'';
	}

/**
* Convert a mm/dd/YYYY date to unix timestamp
* 
* @param string $date
* @param boolean $fail_safe true - returns time(); false - returns ''
* @param mixed $start_of_day true - start of the day; false - end of the day
* @return integer unix timestamp
*/
function convert_date($date, $fail_safe=false, $start_of_day=false)
	{
	$errors=($fail_safe)?time():'';
	if (!$date) { return $errors; }
	if (substr_count($date, '/')!=2) { return $errors; }
	if (strlen($date)!=10) { return $errors; }

	$date=explode('/', $date);

	if ($date[0]<1||$date[0]>12) { return $errors; }
	if ($date[1]<1||$date[1]>31) 
		{
		// take care of 28 vs. 31 days and leap year here soon.
		return $errors; 
		}
	if ($date[2]<1970||$date[2]>2038) { return $errors; } // http://en.wikipedia.org/wiki/Year_2038_problem - how would you handle it? solidphp@gmail.com

	$H=23;
	$i=$s=59;
	if ($start_of_day)
		{
		$H=00;
		$i=$s=00;
		}

	return mktime($H, $i, $s, $date[0], $date[1], $date[2]);
	}

/**
* Make a unique ID
* 
* @param string $prefix
* @param integer $chars
* @return string 
*/
function uid($prefix, $chars=8)
	{
	return $prefix.substr(md5(microtime()), 0, $chars);
	}

/**
* Build a unique profile key string
* 
* @param string $id
* @return string 
*/
function spk($id)
	{
	return x('mod').'|'.x('task')."|{$id}";
	}

/**
* Retrieve a profile value
* 
* @param array $profile
* @param mixed $id
* @return array 
*/
function pk($profile, $id)
	{ 
	return $profile[spk($id)];
	}


/**
* Format a href links 
* 
* @param string $mod
* @param string $task
* @param string $errors
* @param array $extras For use later when SEF comes into play
* @param array $no_redirect
* @return boolean
*/
function redirect($mod, $task, $errors, $extras=array(), $no_redirect=false)
	{
	// set the update notice
	set_cookie('errors', base64_encode(serialize($errors)));

	if ($no_redirect) { return true; }

	die(header('Location: '.href($mod, $task, $extras)));
	}


/**
* Process an error message from redirect()
* 
* @param array $errors
* @return array
*/
function process_messages($errors)
	{
	if (!$_COOKIE['errors']) { return $errors; }

	$cookie_errors=unserialize(base64_decode($_COOKIE['errors']));
	set_cookie('errors', 0, -1);

	$errors=(is_array($errors))?$errors:array();

	return array_merge($errors, $cookie_errors);
	}

if (defined('USE_LOCAL_API')&&USE_LOCAL_API)
	{
	/**
	* Display errors in a nice wrapper.
	*
	* @param array $errors
	* @param boolean $echo	
	* @param boolean $width	
	* @return string
	*/
	function display_errors($errors, $echo=true, $width=false) 
		{ 
		if (!is_array($errors)) { return false; }
		if (empty($errors)) { return false; }

		$class='error';
		if ($errors['success'])
			{
			$class='success';
			unset($errors['success']);
			}

		if ($width) { $width="style='width: {$width}'"; }
		
		$errors=implode("<br />\r\n", $errors);
		$errors="<div class='{$class}'{$width}>{$errors}</div><br />"; 

		if ($echo) { echo $errors; }

		return $errors;
		}

	/**
	* Set the focus class
	*
	* @param string $target
	* @param string $current
	* @return string
	*/
	function on($target, $current)
		{
		if (is_array($target))
			{
			unset($target[array_search('announcements', $target)]);
			if (in_array($current, $target))
				{
				return "class='on'";
				}

			return '';
			}

		if ($target==$current) { return "class='on'"; }

		return '';
		}

	/**
	* Create a breadcrumb menu
	* 
	* @param array $trail
	* @return string
	*/
	function breadcrumb($trail=array())
		{
		$link=href(''); $text=w('home', 0);
		$breadcrumb="<a href='{$link}'>{$text}</a>";
		foreach ($trail as $key => $value)
			{
			$text=sx($value);

			if ($key=='#') { return $breadcrumb.=" &raquo; {$text}"; }

			$link=href('listings', array('category_id' => $key)); 
			$breadcrumb.=" &raquo; <a href='{$link}'>{$text}</a>";
			}

		return $breadcrumb;
		}

	/**
	* Apply nl2br article content if enable
	* 
	* @param string $enabled
	* @param string $content
	* @return boolean
	*/
	function do_nl2br($enabled, $content) 
		{ 
		if ($enabled!='Yes') { return $content; }

		// don't include pre tags
		$pre_buffer=array();
		$buffer='';
		$parts=explode('{skip_nl2br}', $content);
		foreach ($parts as $part)
			{
			if (strpos($part, '{/skip_nl2br}')===false) 
				{
				$buffer.=$part;
				continue;
				}

			$pre=explode('{/skip_nl2br}', $part);
			$pre_buffer[$i+=1]=$pre[0];
			$buffer.="{pre_replace_{$i}}{$pre[1]}";
			}

		unset($i);
		$buffer=sx(nl2br($buffer));
		foreach ($pre_buffer as $pre)
			{
			$i+=1;
			$pre=str_replace('<?', '&lt;?', sx($pre));//  echo sx($pre); die;
			$buffer=str_replace("{pre_replace_{$i}}", $pre, $buffer);
			}

		return $buffer; 
		} 
	}
else
	{
	/**
	* Wrapper to display the breadcrumb
	*
	* Set $echo=false to return the buffer
	* Set $echo=true to echo the buffer
	*
	* <code>
	* breadcrumb(array(
	*			array('Manage Admins', 'admin_edit.php'),
	*			array('Andy Rockwell (General)', 0)
	*			),
	*			0);
	* </code>
	*
	* @param array $array
	* @param boolean $echo
	* @return string
	*/
	function breadcrumb($array, $echo=true)
		{
		if (!is_array($array)) { $array=array(); }
		
		// loop the array and sort out elements
		foreach($array as $key => $value)
			{
			// display the "Menu > Location"
			// if ($key>0) { $buffer.=' &raquo; '; }

			$value[0]=ucws($value[0], false);

			// is this a link?
			if ($value[1])
				{
				$buffer.="<a href='{$value[1]}' ";
				$buffer.="title='{$value[0]}' ";
				$buffer.=">";
				}

			// the text
			$buffer.=$value[0];

			// is this a link? close the tag if so
			if ($value[1]) { $buffer.="</a>"; }
			$buffer.=' &raquo; ';
			}

		if (!$echo) { return $buffer; }

		echo $buffer;
		}
	}

/**
* Format a href links 
* 
* @param string $task
* @param array $extras For use later when SEF comes into play
* @param string $controller
* @return array
*//*
function href($task, $extras=array(), $controller='index')
	{
	$querystring='';
	foreach ((array)$extras as $key => $value)
		{
		if ($key=='task'&&$task) { continue; }

		$querystring.="&{$key}={$value}";
		}

	$q=($task||$querystring)?'?':'';
	$task=($task)?"task={$task}":'';

	return "{$controller}.php{$q}{$task}{$querystring}"; 
	}*/

/**
* Format a href links 
* 
* @param string $mod
* @param string $task
* @param array $extras For use later when SEF comes into play
* @param array $no_module
* @param string $controller
* @return string
*/
function href($mod, $task='', $extras=array(), $no_module=false, $controller='')
	{
	if (defined('DEFAULT_STOREFRONT_MODULE')&&DEFAULT_STOREFRONT_MODULE)
		{

		if (is_array($task))
			{ 
			$querystring='';
			foreach ($task as $key => $value)
				{
				if ($key=='task'&&$task) { continue; }

				$querystring.="&{$key}={$value}";
				}
			}
// 		else if (is_array($extras))
// 			{ 
// 			$querystring="task={$task}";
// 			foreach ($extras as $key => $value)
// 				{
// 				if ($key=='task'&&$task) { continue; }
// 
// 				$querystring.="&{$key}={$value}";
// 				}
// 			}

		$q=($mod||$querystring)?'?':'';
		$mod=($mod)?"task={$mod}":'';

		return  "index.php{$q}{$mod}{$querystring}";
		return "{$extras}.php{$q}{$mod}{$querystring}"; // controller not used anywhere??
		}

	$querystring='';
	foreach ((array)$extras as $key => $value)
		{
		// not valid for the admin area because of the back tabs << at some locations
		//if ($key=='task'&&$task) { continue; }

		$querystring.="&{$key}={$value}";
		}

	$controller=($controller)?$controller:'index';

	return ($no_module)?"{$controller}.php?task={$task}{$querystring}":"index.php?mod={$mod}&task={$task}{$querystring}"; 
	}

/**
* Sort column
* 
* @param integer $sort
* @param integer $default
* @return integer
*/
function sort_column($sort, $default)
	{
	// set the default if no valid sort is passed in
	$sort=($sort)?$sort:1;

	// find the opposite of $sort
	$alt_sort=ooe($sort)?($sort-1):($sort+1);
	
	// confirm we are in the $default range
	if (!in_array($default, array($sort, $alt_sort)))
		{
		// we are not, return the default
		return $default;
		}
	
	// we are in range, return the opposite to $sort
	return $alt_sort;
	}

/**
* Parse the js file application.js to implement localization
* 
* @param string $path
* @return string
*/
function localize_js($path)
	{
	$contents=file_get_contents($path);
	foreach ($GLOBALS['conf']['localization'] as $key => $value)
		{
		$contents=str_replace('{'.$key.'}', $value, $contents);
		}
	return "<script>\n{$contents}\n</script>";
	}

/**
* Shorten a string
* 
* @param string $string	
* @param integer $length
* @param boolean $clip	
* @return string
*/
function shorten_string($string, $length=50, $clip=true)
	{
	if (($strlen=strlen($string))<=$length) { return $string; }

	if (!$clip)
		{
		return substr($string, 0, $length);
		}

	$length-=6;

	$length=floor($length/2);

	$start=substr($string, 0, $length);
	$middle="...";
	$end=substr($string, -$length);

	return "{$start}{$middle}{$end}";
	}

/**
* Fetch an RSS feed
* 
* @param string $url	
* @return string
*/
function fetch_url($url)
	{
	$curl = curl_init();
	
	// Setup headers - I used the same headers from Firefox version 2.0.0.6
	// below was split up because php.net said the line was too long. :/
	$header[0] = "Accept: text/xml,application/xml,application/xhtml+xml,";
	$header[0] .= "text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5";
	$header[] = "Cache-Control: max-age=0";
	$header[] = "Connection: keep-alive";
	$header[] = "Keep-Alive: 300";
	$header[] = "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7";
	$header[] = "Accept-Language: en-us,en;q=0.5";
	$header[] = "Pragma: "; // browsers keep this blank.
	
	curl_setopt($curl, CURLOPT_URL, $url);
	curl_setopt($curl, CURLOPT_USERAGENT, 'Googlebot/2.1 (+http://www.google.com/bot.html)');
	curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
	curl_setopt($curl, CURLOPT_REFERER, 'http://www.google.com');
	curl_setopt($curl, CURLOPT_ENCODING, 'gzip,deflate');
	curl_setopt($curl, CURLOPT_AUTOREFERER, true);
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($curl, CURLOPT_TIMEOUT, 10);
	
	$html = curl_exec($curl); // print_R($html); die; // execute the curl command

	curl_close($curl); // close the connection
	
	return $html; // and finally, return $html
	}

/**
* Fetch an RSS feed
* 
* @param string $url	
* @return boolean
*/
function fetch_rss($url)
	{
	return fetch_url($url);
	}

/**
* Is the $number odd or even?
* 
* @param integer $number	
* @return boolean
*/
function ooe($number) { return ($number%2)==0; } 


/**
* Toggle state
* 
* @param integer $state	
* @return boolean
*/
function ts($state) { return ($state)?false:true; } 

/**
* Make something like this: license_management
* Look like this: License Management
* 
* @param string $plain_text	
* @param boolean $echo	
* @return string
*/
function ucws($plain_text, $echo=true)
	{
	$formatted_text=ucwords(implode(" ", explode('_', $plain_text)));

	if (!$echo) 
		{
		return $formatted_text;
		}

	echo $formatted_text;
	}

/**
* Resize an image 
* 
* @param string $name		Original filename
* @param string $filename	Filename of the resized image
* @param integer $new_w		Width of resized image
* @param integer $new_h		Height of resized image
* @return image on success; false for failure 
*/
function resize($name, $filename, $new_w, $new_h)
	{
	$src_img=false;
	$extension=get_file_extenstion($name);
	if ($extension==('jpg'||'jpeg')) { $src_img=imagecreatefromjpeg($name); }
	if ($extension=='png') { $src_img=imagecreatefromjpeg($name); }
	if ($extension=='gif') { $src_img=imagecreatefromgif($name); }
	if (!$src_img) { return false; }

	$old_x=imageSX($src_img);
	$old_y=imageSY($src_img);

	if ($old_x > $old_y) 
		{
		$thumb_w=$new_w;
		$thumb_h=$old_y*($new_h/$old_x);
		}

	if ($old_x < $old_y) 
		{
		$thumb_w=$old_x*($new_w/$old_y);
		$thumb_h=$new_h;
		}

	if ($old_x == $old_y) 
		{
		$thumb_w=$new_w;
		$thumb_h=$new_h;
		}

	$dst_img=ImageCreateTrueColor($thumb_w,$thumb_h);
	imagecopyresampled($dst_img,$src_img,0,0,0,0,$thumb_w,$thumb_h,$old_x,$old_y); 

	if (preg_match("/png/",$system[1]))
		{
		imagepng($dst_img,$filename); 
		} 
	else { imagejpeg($dst_img,$filename); }

	imagedestroy($dst_img); 
	imagedestroy($src_img); 
	}

/**
* Parse the debian mime.types file
* 
* @param boolean $mime 
* @param boolean $ext 
* @return array 
*/
function mime_types($mime=true, $ext=true)
	{
	$buffer=array();
	$types=file('modules/common/mime.types'); // pr($types);
	foreach ($types as $key => $value)
		{
		if ($key<26) { unset($types[$key]); continue; }
		if (strlen(trim($value))==0) { unset($types[$key]); continue; }

		$parts=preg_split("/\s/",trim($value));

		if ($mime&&$ext)
			{
			if (count($parts)<=1)
				{
				$buffer[]=array('mime' => $parts[0], 'ext' =>  '');
				}
			else
				{
				$mime=$parts[0];
				unset($parts[0]);

				foreach ($parts as $part)
					{
					if (strlen(trim($part))==0) { continue; }
					$buffer[]=array('mime' => $mime, 'ext' =>  $part);
					}
				}
			}
		else if ($mime)
			{
			$buffer[]=$parts[0];
			}
		else if ($ext&&count($parts)>1)
			{
			$buffer[]=array_pop($parts);
			}
		}

	return $buffer;
	}

/**
* Get the mime type for $filename
* 
* @param string $filename
* @return string 
*/
function get_mime_type($filename)
	{
	$filename=strtolower($filename);

	$mime_types=mime_types();
	$this_ext=get_file_extenstion($filename);
	foreach ($mime_types as $value)
		{
		if ($this_ext==$value['ext'])
			{
			return $value['mime'];
			}
		}

	return '';
	}

/**
* Parse for a file extension
* 
* @param string $filename
* @return string 
*/
function get_file_extenstion($filename)
	{
	$filename=array_pop(explode('/', $filename));

	$ext=substr($filename, strpos($filename, '.')+1);
	if (!$ext)
		{
		$filename=array_pop(explode('\\', $file));
		$ext=substr($file, strpos($filename, '.')+1);
		}

	return $ext;
	}


/**
* All possible months.
* 
* @return array
*/
function months()
	{
	return array(
				'01' => w('january', 0),
				'02' => w('february', 0),
				'03' => w('march', 0),
				'04' => w('april', 0),
				'05' => w('may', 0),
				'06' => w('june', 0),
				'07' => w('july', 0),
				'08' => w('august', 0),
				'09' => w('september', 0),
				'10' => w('october', 0),
				'11' => w('november', 0),
				'12' => w('december', 0)
				);
	}

/**
* All possible days (integer).
* 
* @return array
*/
function days()
	{
	$buffer=array();
	for($i=1; $i<=31; $i++)
		{
		$buffer[$i]=sprintf('%02.0f', $i);
		}

	return $buffer;
	}

/**
* All possible days (integer).
* 
* @return array
*/
function years()
	{
	$start=date('Y')-5;
	$end=date('Y')+5;

	$buffer=array();
	for($i=$start; $i<=$end; $i++)
		{
		$buffer[$i]=sprintf('%04.0f', $i);
		}

	return $buffer;
	}

/**
* All possible hours
* 
* @return array
*/
function hours()
	{
	$buffer=array();
	for ($h=1; $h<=11; $h++)
		{
		$buffer["{$h}AM"]="{$h}:00 AM";
		}

	$buffer["12PM"]="12:00 PM";

	for ($h=1; $h<=11; $h++)
		{
		$buffer["{$h}PM"]="{$h}:00 PM";
		}

	$buffer["12AM"]="12:00 AM";

	return $buffer;
	}

/**
* Display an error message on the page
*
* @param string $message The message to convey
* @param string $type Notice, Warning, Error, etc.
* @return string formatted message
*/
function message($message, $type, $echo=true)
	{
	$buffer="<div id='notice'><b>{$type}</b> {$message}</div>";

	if (!$echo) { return $buffer; }

	echo $buffer;
	}

/**
* Shorten image source length
* 
* @param string $filename
* @param boolean $echo
* @return string
*/
function skin_image($filename, $echo=true)
	{
	$path='skins/'.config('skin')."/main/images/{$filename}";
	
	if (!$echo) { return $path; }

	echo $path;
	}

/**
* Shorten image source length
* 
* @param string $filename
* @param boolean $echo
* @return string
*/
function skin_icon($filename, $echo=true)
	{
	$path='skins/'.config('skin')."/main/images/icons/{$filename}";
	
	if (!$echo) { return $path; }

	echo $path;
	}

/**
* Parse XML using eval() to compile the resulting array
*
* @param string XML to parse
* @return array parsed XML
*/
function parse_xml_by_eval($xml)
	{
	$parser=@xml_parser_create('');
	@xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	@xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	@xml_parse_into_struct($parser, $xml, $values, $tags);
	@xml_parser_free($parser);

	$hash_stack=array();

	foreach($values as $key => $val)
		{
		switch ($val['type'])
			{
			case "open":
			array_push($hash_stack, $val['tag']);
			break;

			case "close":
			array_pop($hash_stack);
			break;

			case "complete":
			array_push($hash_stack, $val['tag']);

			# uncomment to see what this function is doing
			# echo("\$ret[" . implode($hash_stack, "][") . "] = '{$val[value]}';\n");

			eval("\$ret[".implode($hash_stack, "][")."]='{$val[value]}';");
			array_pop($hash_stack);
			break;
			}
		}

	return $ret;
	}

/**
* creates/updates the API key
*
* @return string
*/
function change_api_key()
	{
	$api_key=md5(time().'10,000 Acres.');

	if (!array_key_exists('api_key', config()))
		{
		$query=array();
		$query['data']=array('config_key' => 'api_key',
							 'config_value' => $api_key);
		$query['table']=acb().'_fw_config';

		sql::insert_record($query);

		return $api_key;
		}

	$query=array();
	$query['data']=array('config_value' => $api_key);
	$query['table']=acb().'_fw_config';
	$query['where']=array(array('config_key', '=', 'api_key'));

	sql::update_record($query);

	return $api_key;
	}


/**
* Removes the trailing slash from a URL
*
* @param string $url
* @return string
*/
function sts($url)
	{
	if (substr($url, -1)=="/")
		{
		$url=substr($url, 0, strlen($url)-1);
		}

	return $url;
	}



/**
* Query the configuration into a single array
*
* @param array $config Only database connections in $config at this point.
* @return array
*/
function compile_config($config)
	{
	$get_config=array();
	$get_config['data']=array('config_key', 'config_value');
	$get_config['table']=acb().'_fw_config';
	$get_config=sql::select_records($get_config);
	if (!$get_config['data']) { die($GLOBALS['w']['globals']['no_configuration_found']); }

	foreach ($get_config['data'] as $temp)
		{
		if (!is_array($config)) { $config=array(); }
		$config=array_merge($config, array($temp['config_key'] => $temp['config_value']));
		}
	
	if (!$config['skin']) { $config['skin']='Default'; }

	return $config;
	}

/**
* Returns $config from the global scope
*
* @param mixed $which The key to return or boolean false.
* @param boolean $echo
* @return string
*/
function config($which=false, $echo=false) 
	{ 
	if ($which&&$GLOBALS['conf'][$which]) 
		{ 
		if ($echo) 
			{ 
			echo $GLOBALS['conf'][$which];
			return '';
			}

		return $GLOBALS['conf'][$which];
		}

	if ($echo) 
		{ 
		echo $GLOBALS['conf'];
		return '';
		}

	return $GLOBALS['conf']; 
	}



/**
* Get the current file name with or without extension
*
* Set $full=false to return file name only
* Set $full=true to return the full file name + extension
*
* @param boolean $full
* @return string
*/
function file_name($full=false)
	{
	$first=strrpos($_SERVER['PHP_SELF'], '/')+1;
	$length=strlen($_SERVER['PHP_SELF']);
	$file=substr($_SERVER['PHP_SELF'], $first, $length-$first);

	if ($full) { return $file; }

	return substr($file, 0, strpos($file, '.'));
	}


/**
* Localization wrapper function based on $GLOBALS['conf']['localization']
*
* @param string $mask
* @param boolean $echo
* @return string
*/
function w($mask, $echo=true)
	{
	$translation=$GLOBALS['conf']['localization'][$mask];
	// $translation=($translation)?$translation:$GLOBALS['conf']['localization']['no_translation_found'];
	$translation=($translation)?$translation:$mask; // more useful to know the tag, added for v1.9

	if (!$echo) { return s($translation); }

	echo s($translation);
	}


/**
* Display an array key formatted.
*
* example $array['menu_name'] -> Menu Name
*
* @param string $key
* @return string
*/
function key2display($key)
	{
	// replace _ with spaces
	$key=str_replace('_', ' ', $key);

	// create an array of all words in the key
	$parts=explode(' ', $key);

	// make each word upper case
	$buffer=array();
	foreach ($parts as $part)
		{
		$buffer[]=ucfirst($part);
		}

	// glue it back together as a string
	return implode(' ', $buffer);
	}


/**
* Debug function
*
* Set $echo=false to return the buffer
* Set $echo=true to die() the buffer
* Set a $GLOBALS['log_it'] to write to the filesystem
*
* @param array $array
* @param boolean $echo
* @return string
*/
function pr($array, $echo=true)
	{
	if ($GLOBALS['log_it'])
		{
		$fp=fopen('/home/andy/public_html/log.txt', 'w');
		fwrite($fp, var_export($array, 1));
		fclose($fp);
		}

	$buffer="<pre>\n";
	$buffer.=var_export($array, 1);
	$buffer.="\n</pre>";

	if ($echo) { die($buffer); }

	return $buffer;
	}


/**
* Get the current web path
*
* @return string
*/
function web_path()
	{
	$web_path=$_SERVER['HTTP_HOST'];
	$web_path.=str_replace($_SERVER['DOCUMENT_ROOT'], '', path_translated());
	
	if (substr(strtolower($web_path), 0, 7)!='http://') { return 'http://'.$web_path; }
	
	return $web_path;
	}


/**
* Linux and windows path translated
*
* @return string
*/
function path_translated()
	{
	if ($_SERVER['PATH_TRANSLATED'])
		{
		return @substr($_SERVER['PATH_TRANSLATED'], 0, @strrpos($_SERVER['PATH_TRANSLATED'], "/"));
		}

	if ($_SERVER['SCRIPT_FILENAME'])
		{
		return @substr($_SERVER['SCRIPT_FILENAME'], 0, @strrpos($_SERVER['SCRIPT_FILENAME'], "/"));
		}

	return @substr($_SERVER['ORIG_PATH_TRANSLATED'], 0, @strrpos($_SERVER['ORIG_PATH_TRANSLATED'], "\\"));
	}

/**
* Display errors in a nice wrapper.
*
* @param array $errors
* @param boolean $echo
* @param boolean $width
* @param boolean $shake
* @return string
*/
function errors($errors, $echo=true, $width=false, $shake=true) 
	{ 
	if (!is_array($errors)||empty($errors)) { return ''; }
	if ($width) { $width="style='width: {$width}'"; }

	$which=($errors['type']=='success')?'success':'error';
	unset($errors['type']);


	$id='x'.substr(md5(microtime().microtime()), -7);
	$errors=implode("<br />\r\n", $errors);
	$click_to_close=w('click_to_close', 0);
	$errors="<div onclick='\$(this).hide();' id='{$id}' title='{$click_to_close}' class='{$which}'{$width}>{$errors}</div>"; 
	$errors.="<input type='hidden' id='error_div_id[]' value='{$id}' />"; 

	if ($echo) { echo $errors; }

	return $errors;
	}

/**
* Get the cookie domain
*
* @return string
*/
function cookie_domain()
	{
	if (defined('use_cookie_domain')) { return use_cookie_domain; }

	$host=($_SERVER['HTTP_X_FORWARDED_HOST'])?$_SERVER['HTTP_X_FORWARDED_HOST']:$_SERVER['HTTP_HOST'];

	// if it's an IP address just return the IP
	if (eregi('^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}(/[0-9]{1,2}){0,1}$', $host))
		{
		return $host;
		}

	if (strpos($host, '.')===false) { return ''; }

	if (in_array($snip=substr($host, 0, 4), array('WWW.', 'www.')))
		{
		$host=str_replace($snip, '', $host);
		}

	return ".{$host}";
	}

/**
* Wrapper to handle cookies
*
* @param string $name
* @param mixed $data
* @param integer $expires
* @return boolean
*/
function set_cookie($name, $data, $expires=0)
	{
	return @setcookie($name, $data, $expires, '/', cookie_domain());
	}


/**
* Grab the IP address of the end user
*
* @return string
*/
function capture_remote()
	{
	if ($_SERVER["HTTP_X_FORWARDED_FOR"]) { return $_SERVER["HTTP_X_FORWARDED_FOR"]; }

	if ($_SERVER["HTTP_CLIENT_IP"]) {  return $_SERVER["HTTP_CLIENT_IP"]; }

	return $_SERVER["REMOTE_ADDR"];
	}


/**
* Clears the cache to ensure fresh data
*/
function clear_cache()
	{	
	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
	header("Cache-Control: no-store, no-cache, must-revalidate");
	header("Cache-Control: post-check=0, pre-check=0", false);
	// header('Content-type: text/html; charset=utf-8');
	header("Pragma: no-cache");
	}


/**
* Prep data for the database
*
* @param string $var
* @return string
*/
function a($var) { return addslashes(trim(@htmlentities($var, ENT_QUOTES))); }


/**
* Convert data back to state before a()
*
* @param string $var
* @return string Leave both double and single quotes unconverted.
*/
function s($var) { return stripslashes(trim(html_entity_decode($var, ENT_NOQUOTES)));}


/**
* Convert data back to state before a()
*
* @param string $var
* @return string Convert both double and single quotes..
*/
function sx($var) 
	{
	if (is_windows($var)) { return stripslashes($var); } // commenting this out for v1.9. Why was it added???
	return stripslashes(trim(html_entity_decode($var, ENT_QUOTES))); 
	}

/**
* Allows you to find a value in $_GET, $_POST and then $_COOKIE.
*
* @param string $target The array key.
* @param boolean $all Return the entire array
* @return mixed Returns false on nothing found, returns the value on something found
*/
function x($target, $all=false, $priority='pgc')
	{
	while (strlen($priority)) 
		{
		$p = substr($priority,0,1);
		if ($p=='g' && isset($_GET[$target])) { return ($all)?$_GET:$_GET[$target]; }
		if ($p=='p' && isset($_POST[$target])) { return ($all)?$_POST:$_POST[$target]; }
		if ($p=='c' && isset($_COOKIE[$target])) { return ($all)?$_COOKIE:$_COOKIE[$target]; }
		$priority=substr($priority,1);
		}

	return false;
	}

/**
* Shows the copyright date
*
* @param integer $start The year to start with
* @param string $spacer "-" by default
* @return string year_start to year_current
*/
function show_copyright_date($start=2003, $spacer=' - ')
	{
	$current=date('Y');

	if ($current>$start) { return $start.$spacer.$current; }
	
	return $current;
	}

/**
* UTF8 Helper
*
* @access private
*/
function charset_decode_utf_8 ($string) {
      /* Only do the slow convert if there are 8-bit characters */
    /* avoid using 0xA0 (\240) in ereg ranges. RH73 does not like that */
    if (! ereg("[\200-\237]", $string) and ! ereg("[\241-\377]", $string))
        return $string;

    // decode three byte unicode characters
    $string = preg_replace("/([\340-\357])([\200-\277])([\200-\277])/e",       
    "'&#'.((ord('\\1')-224)*4096 + (ord('\\2')-128)*64 + (ord('\\3')-128)).';'",   
    $string);

    // decode two byte unicode characters
    $string = preg_replace("/([\300-\337])([\200-\277])/e",
    "'&#'.((ord('\\1')-192)*64+(ord('\\2')-128)).';'",
    $string);

    return $string;
} 

/**
* Processes cookies and sessions returned by 3rd party loginshare modules
*
* @param array $third_party
* @return boolean 
*/
function process_loginshare($third_party)
	{
	foreach ((array)$third_party as $key => $run)
		{
		if ($run['session'])
			{
			if (isset($_SESSION)) { session_start(); }
			$_SESSION=unserialize(base64_decode($run['session']));
			}

		if ($run['cookie'])
			{
			$cookies=unserialize(base64_decode($run['cookie']));				
			foreach ((array)$cookies as $cookie_key => $cookie_value)
				{
				set_cookie($cookie_key, $cookie_value);
				}
			}
		}

	return true;
	}
?>