<?PHP
/**
 * SPBAS Storefront General Use Functions
 *
 * @package    General_Use_Functions_Storefront
 * @author     Andy Rockwell <support@solidphp.com>
 */


/**
* Generate a random hex color
* 
* @return string
*/
function get_random_color()
	{
	for ($i = 0; $i<6; $i++)
		{
		$c.= dechex(rand(0,15));
		}

	return $c;
	} 


/**
* Is the $number odd or even?
* 
* @param integer $number	
* @return boolean
*/
function ooe($number) { return ($number%2)==0; } 

/**
* Use https or not
* 
* @param array $url
* @return string
*/
function url($url)
	{
	$url=explode('//', $url);
	return (($_SERVER['HTTPS'])?'https://':'http://').$url[1];
	} 

/**
* Create a breadcrumb menu
* 
* @param array $trail
* @return string
*/
function breadcrumb($trail=array())
	{
	$link=href(''); $text=w('home', 0);
	$breadcrumb="<a href='{$link}'>{$text}</a>";
	foreach ($trail as $key => $value)
		{
		$text=sx($value);

		if ($key=='#') { return $breadcrumb.=" &raquo; {$text}"; }

		$link=href('listings', array('category_id' => $key)); 
		$breadcrumb.=" &raquo; <a href='{$link}'>{$text}</a>";
		}

	return $breadcrumb;
	}

/**
* Grab the IP address of the end user
*
* @return mixed
*/
function capture_remote()
	{
	if ($_SERVER["HTTP_X_FORWARDED_FOR"]) { return $_SERVER["HTTP_X_FORWARDED_FOR"]; }

	if ($_SERVER["HTTP_CLIENT_IP"]) {  return $_SERVER["HTTP_CLIENT_IP"]; }

	return $_SERVER["REMOTE_ADDR"];
	}

/**
* Apply nl2br article content if enable
* 
* @param string $enabled
* @param string $content
* @return boolean
*/
function do_nl2br($enabled, $content) 
	{ 
	if ($enabled!='Yes') { return $content; }

	// don't include pre tags
	$pre_buffer=array();
	$buffer='';
	$parts=explode('{skip_nl2br}', $content);
	foreach ($parts as $part)
		{
		if (strpos($part, '{/skip_nl2br}')===false) 
			{
			$buffer.=$part;
			continue;
			}

		$pre=explode('{/skip_nl2br}', $part);
		$pre_buffer[$i+=1]=$pre[0];
		$buffer.="{pre_replace_{$i}}{$pre[1]}";
		}

	unset($i);
	$buffer=sx(nl2br($buffer));
	foreach ($pre_buffer as $pre)
		{
		$i+=1;
		$pre=str_replace('<?', '&lt;?', sx($pre));//  echo sx($pre); die;
		$buffer=str_replace("{pre_replace_{$i}}", $pre, $buffer);
		}

	return $buffer; 
	} 

/**
* Post XML using cURL
* 
* @param integer $url
* @param integer $xml
* @return boolean
*/
function post_xml($url, $xml) 
	{
	$ch=curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt ($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
	$result=curl_exec($ch);
	curl_close($ch);

	return $result;
	}

/**
* Recursively search for the configuration file!
* 
* @return string 
*/
function start_m3()
	{
	$try=''; 
	while (strlen($try)<=3)
		{
		$conf="{$try}m3_configuration.php";
		if (file_exists($conf)) {  return $conf; }
		$try.='../';
		}

	return false;
	}

/**
* Verify the url entered is valid.
* 
* @param string $url
* @param string $mime_type
* @return boolean
*/
function test_url($url, $mime_type='image/x-icon')
	{
	return true;

	$curl=curl_init("{$url}/favicon.ico");
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
	curl_exec($curl);

	if (curl_errno($curl))
		{
		return false;
		}
	$i=curl_getinfo($curl);
	curl_close($curl);

	return ($i['http_code']==200)?true:false;
	// return ($i['http_code']==200&&$i['content_type']==$mime_type)?true:false;
	}

/**
* Convert a Comma Separated Value list into an array
* 
* @param string $sql
* @return array
*/
function parse_sql($sql)
	{
	$buffer='';
	$results=array();
	foreach ($sql as $sql)
		{
		$sql=trim($sql);
		if (substr($sql, 0, 1)==" ") { continue; }
		if (substr($sql, 0, 2)=="--") { continue; }
		if (!$sql) { continue; }

		if (substr($sql, -1)==";")
			{
			$buffer.=$sql;
			$buffer=substr($buffer, 0, strlen($buffer)-1);
			# $buffer=str_replace("`", "", $buffer);

			$results[]=s($buffer);
			$buffer='';
			}
		else
			{
			$buffer.=$sql;
			}
		}

	return $results;
	}

/**
* Fetch an RSS feed
* 
* @param string $url	
* @return boolean
*/
function fetch_rss($url)
	{
	$curl = curl_init();
	
	// Setup headers - I used the same headers from Firefox version 2.0.0.6
	// below was split up because php.net said the line was too long. :/
	$header[0] = "Accept: text/xml,application/xml,application/xhtml+xml,";
	$header[0] .= "text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5";
	$header[] = "Cache-Control: max-age=0";
	$header[] = "Connection: keep-alive";
	$header[] = "Keep-Alive: 300";
	$header[] = "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7";
	$header[] = "Accept-Language: en-us,en;q=0.5";
	$header[] = "Pragma: "; // browsers keep this blank.
	
	curl_setopt($curl, CURLOPT_URL, $url);
	curl_setopt($curl, CURLOPT_USERAGENT, 'Googlebot/2.1 (+http://www.google.com/bot.html)');
	curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
	curl_setopt($curl, CURLOPT_REFERER, 'http://www.google.com');
	curl_setopt($curl, CURLOPT_ENCODING, 'gzip,deflate');
	curl_setopt($curl, CURLOPT_AUTOREFERER, true);
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($curl, CURLOPT_TIMEOUT, 10);
	
	$html = curl_exec($curl); // print_R($html); die; // execute the curl command
	curl_close($curl); // close the connection
	
	return $html; // and finally, return $html
	}

/**
* Convert date to unix timestamp
*
* @param integer $month
* @param integer $day
* @param integer $year
* @return integer
*/
function to_unixtime($month, $day, $year)
	{
	return mktime(0, 0, 0, $month, $day, $year);
	}


/**
* Shuffle an array
*
* @param array $array
* @param integer $total
* @return integer
*/
function fisherYatesShuffle(&$array, $total)
	{
	for ($i=0; $i<$total; $i++)
		{
		$j=@mt_rand(0, $i);
		$temp=$array[$i];
		$array[$i]=$array[$j];
		$array[$j]=$temp;
		}
	
	return $array;
	}


/**
* Returns $config from the global scope
*
* @param mixed $which The key to return or boolean false.
* @param boolean $echo	
* @return mixed
*/
function config($which=false, $echo=false) 
	{ 
	if ($which&&$GLOBALS['conf']['site'][$which]) 
		{ 
		if ($echo) 
			{ 
			echo $GLOBALS['conf']['site'][$which];
			return '';
			}

		return $GLOBALS['conf']['site'][$which];
		}

	if ($which&&$GLOBALS['conf'][$which]) 
		{ 
		if ($echo) 
			{ 
			echo $GLOBALS['conf'][$which];
			return '';
			}

		return $GLOBALS['conf'][$which];
		}

	if ($echo) 
		{ 
		echo $GLOBALS['conf'];
		return '';
		}

	return $GLOBALS['conf']; 
	}

/**
* Enforce the use of www. or no www. 
* 
* @param string $host
* @param string $rules
* @return string
*/
function enforce_www($host, $rules)
	{
	if ($rules=='disable') { return false; }

	$isset_www=strpos($host, 'www.');
	if ($isset_www===0&&$rules=='no_www')
		{
		header('HTTP/1.1 301 Moved Permanently');
		die(header('Location: http://'.substr($host, 4)));
		}

	if ($isset_www===false&&$rules=='www')
		{
		header('HTTP/1.1 301 Moved Permanently');
		die(header("Location: http://www.{$host}"));
		}

	return false;
	}

/**
* Format a href links 
* 
* @param string $task
* @param array $extras For use later when SEF comes into play
* @param string $controller
* @return array
*/
function href($task, $extras=array(), $controller='index')
	{
	$rewrite=config('mod_rewrite');

	if ($rewrite&&$rewrite=='Enable'&&$extras['seo_page_name'])
		{
		return $extras['seo_page_name'];
		}
	else { unset($extras['seo_page_name']); }

	$querystring='';
	foreach ($extras as $key => $value)
		{
		if ($key=='task'&&$task) { continue; }

		$querystring.="&{$key}={$value}";
		}

	$q=($task||$querystring)?'?':'';
	$task=($task)?"task={$task}":'';

	return "{$controller}.php{$q}{$task}{$querystring}"; 
	}


/**
* Set the focus class
*
* @param string $target
* @param string $current
* @return string
*/
function on($target, $current)
	{
	if (is_array($target))
		{
		unset($target[array_search('announcements', $target)]);
		if (in_array($current, $target))
			{
			return "class='on'";
			}

		return '';
		}

	if ($target==$current) { return "class='on'"; }

	return '';
	}


/**
* Localization wrapper function based on $GLOBALS['conf']['localization']
*
* @param string $mask
* @param boolean $echo
* @return string
*/
function w($mask, $echo=true)
	{
	$translation=$GLOBALS['conf']['localization'][$mask];
	$translation=($translation)?$translation:$GLOBALS['conf']['localization']['no_translation_found'];

	if (!$echo) { return sx($translation); }

	echo sx($translation);
	}

/**
* All possible months.
* 
* @return array
*/
function months()
	{
	return array(
			'01' => 'January',
			'02' => 'February',
			'03' => 'March',
			'04' => 'April',
			'05' => 'May',
			'06' => 'June',
			'07' => 'July',
			'08' => 'August',
			'09' => 'September',
			'10' => 'October',
			'11' => 'November',
			'12' => 'December'
			);
	}

/**
* All possible days (integer).
* 
* @return array
*/
function years()
	{
	$start=date('Y');
	$end=date('Y')+5;

	$buffer=array();
	for($i=$start; $i<=$end; $i++)
		{
		$buffer[$i]=sprintf('%04.0f', $i);
		}

	return $buffer;
	}


/**
* Debug helper - prints a formatted array
* 
* @param array $stack The array to display
* @param boolean $stop_execution
* @return string 
*/
function pr($stack, $stop_execution=true)
	{
	$formatted='<pre>'.var_export((array)$stack, 1).'</pre>';

	if ($stop_execution) { die($formatted); }

	return $formatted;
	}

/**
* Clear cache for a session
*/
function clear_cache()
	{
	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
	header("Cache-Control: no-store, no-cache, must-revalidate");
	header("Cache-Control: post-check=0, pre-check=0", false);
	// header('Content-type: text/html; charset=utf-8');
	header("Pragma: no-cache");
	}

/**
* Display errors in a nice wrapper.
*
* @param array $errors
* @param boolean $echo	
* @param boolean $width	
* @return string
*/
function display_errors($errors, $echo=true, $width=false) 
	{ 
	if (!is_array($errors)) { return false; }
	if (empty($errors)) { return false; }

	$class='error';
	if ($errors['success'])
		{
		$class='success';
		unset($errors['success']);
		}

	if ($width) { $width="style='width: {$width}'"; }
	
	$errors=implode("<br />\r\n", $errors);
	$errors="<div class='{$class}'{$width}>{$errors}</div><br />"; 

	if ($echo) { echo $errors; }

	return $errors;
	}

/**
* UTF8 Helper
*
* @access private
*/
function charset_decode_utf_8 ($string) {
      /* Only do the slow convert if there are 8-bit characters */
    /* avoid using 0xA0 (\240) in ereg ranges. RH73 does not like that */
    if (! ereg("[\200-\237]", $string) and ! ereg("[\241-\377]", $string))
        return $string;

    // decode three byte unicode characters
    $string = preg_replace("/([\340-\357])([\200-\277])([\200-\277])/e",       
    "'&#'.((ord('\\1')-224)*4096 + (ord('\\2')-128)*64 + (ord('\\3')-128)).';'",   
    $string);

    // decode two byte unicode characters
    $string = preg_replace("/([\300-\337])([\200-\277])/e",
    "'&#'.((ord('\\1')-192)*64+(ord('\\2')-128)).';'",
    $string);

    return $string;
} 

/**
* Get the cookie domain
*
* @return string
*/
function cookie_domain()
	{
	$cookie_domain=config('use_cookie_domain');
	if (!is_array($cookie_domain)&&$cookie_domain) { return $cookie_domain; }

	$host=($_SERVER['HTTP_X_FORWARDED_HOST'])?$_SERVER['HTTP_X_FORWARDED_HOST']:$_SERVER['HTTP_HOST'];

	// if it's an IP address just return the IP
	if (eregi('^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}(/[0-9]{1,2}){0,1}$', $host))
		{
		return $host;
		}

	if (strpos($host, '.')===false) { return ''; }

	if (in_array($snip=substr($host, 0, 4), array('WWW.', 'www.')))
		{
		$host=str_replace($snip, '', $host);
		}

	return ".{$host}";
	}

/**
* Wrapper to handle cookies
*
* @param string $name
* @param mixed $data
* @param integer $expires
* @return boolean
*/
function set_cookie($name, $data, $expires=0)
	{
	return @setcookie($name, $data, $expires, '/', cookie_domain());
	}


/**
* Determine if we are using windows
* 
* @param string $path
* @return boolean
*/
function is_windows($path='')
	{
	if (@strpos($path, '\\')===true) { return true; }
	if (substr(php_uname(), 0, 7)=='Windows') { return true; }
	return false; 
	}


/**
* Prep data for the database
*
* @param string $var
* @return string
*/
function a($var) { return addslashes(trim(@htmlentities($var, ENT_QUOTES))); }


/**
* Convert data back to state before a()
*
* @param string $var
* @return string Leave both double and single quotes unconverted.
*/
function s($var) { return stripslashes(trim(html_entity_decode($var, ENT_NOQUOTES)));}


/**
* Convert data back to state before a()
*
* @param string $var
* @return string Convert both double and single quotes..
*/
function sx($var) 
	{ 
	if (is_windows($var)) { return $var; }
	return stripslashes(trim(html_entity_decode($var, ENT_QUOTES))); 
	}

/**
* Allows you to find a value in $_GET, $_POST and then $_COOKIE.
*
* @param string $target The array key.
* @param boolean $all Return the entire array
* @return mixed Returns false on nothing found, returns the value on something found
*/
function x($target, $all=false, $priority='pgc')
	{
	while (strlen($priority)) 
		{
		$p = substr($priority,0,1);
		if ($p=='g' && isset($_GET[$target])) { return ($all)?$_GET:$_GET[$target]; }
		if ($p=='p' && isset($_POST[$target])) { return ($all)?$_POST:$_POST[$target]; }
		if ($p=='c' && isset($_COOKIE[$target])) { return ($all)?$_COOKIE:$_COOKIE[$target]; }
		$priority=substr($priority,1);
		}

	return false;
	}

/**
* Parse XML using eval() to compile the resulting array
*
* @param string $xml
* @return array parsed XML
*/
function parse_xml_by_eval($xml)
	{
	$parser=@xml_parser_create('');
	@xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	@xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	@xml_parse_into_struct($parser, $xml, $values, $tags);
	@xml_parser_free($parser);

	$hash_stack=array();

	foreach($values as $key => $val)
		{
		switch ($val['type'])
			{
			case "open":
			array_push($hash_stack, $val['tag']);
			break;

			case "close":
			array_pop($hash_stack);
			break;

			case "complete":
			array_push($hash_stack, $val['tag']);

			# uncomment to see what this function is doing
			# echo("\$ret[" . implode($hash_stack, "][") . "] = '{$val[value]}';\n");

			eval("\$ret[".implode($hash_stack, "][")."]='{$val[value]}';");
			array_pop($hash_stack);
			break;
			}
		}

	return $ret;
	}
?>